@@ -27,18 +27,10 @@
 #include "FbPixmap.hh"
 #include "GContext.hh"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif //HAVE_CONFIG_H
-
 #ifdef HAVE_SETLOCALE
 #include <locale.h>
 #endif // HAVE_SETLOCALE
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif // _GNU_SOURCE
-
 #ifdef HAVE_CSTDIO
   #include <cstdio>
 #else
@@ -59,56 +51,6 @@ using std::string;
 
 namespace {
 
-const char *getFontSize(const char *pattern, int *size) {
-    const char *p;
-    const char *p2=0;
-    int n=0;
-
-    for (p=pattern; 1; p++) {
-        if (!*p) {
-            if (p2!=0 && n>1 && n<72) {
-                *size = n; return p2+1;
-            } else {
-                *size = 16; return 0;
-            }
-        } else if (*p=='-') {
-            if (n>1 && n<72 && p2!=0) {
-                *size = n;
-                return p2+1;
-            }
-            p2=p; n=0;
-        } else if (*p>='0' && *p<='9' && p2!=0) {
-            n *= 10;
-            n += *p-'0';
-        } else {
-            p2=0; n=0;
-        }
-    }
-}
-
-const char *getFontElement(const char *pattern, char *buf, int bufsiz, ...) {
-    const char *p, *v;
-    char *p2;
-    va_list va;
-
-    va_start(va, bufsiz);
-    buf[bufsiz-1] = 0;
-    buf[bufsiz-2] = '*';
-    while((v = va_arg(va, char *)) != 0) {
-        p = FbTk::StringUtil::strcasestr(pattern, v);
-        if (p) {
-            strncpy(buf, p+1, bufsiz-2);
-            p2 = strchr(buf, '-');
-            if (p2) *p2=0;
-            va_end(va);
-            return p;
-        }
-    }
-    va_end(va);
-    strncpy(buf, "*", bufsiz);
-    return 0;
-}
-
 XFontSet createFontSet(const char *fontname, bool& utf8mode) {
     Display *display = FbTk::App::instance()->display();
     XFontSet fs;
@@ -158,7 +100,8 @@ XFontSet createFontSet(const char *fontname, bool& utf8mode) {
     return fs;
 }
 
-};
+}
+
 namespace FbTk {
 
 XmbFontImp::XmbFontImp(const char *filename, bool utf8) : m_fontset(0), m_setextents(0), m_utf8mode(utf8) {
@@ -188,10 +131,10 @@ bool XmbFontImp::load(const string &fontname) {
     return true;
 }
 
-void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const FbString &text,
+void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const char* text,
                           size_t len, int x, int y, FbTk::Orientation orient) {
 
-    if (m_fontset == 0)
+    if (!text || !*text || m_fontset == 0)
         return;
 
     if (orient == ROT0) {
@@ -199,13 +142,11 @@ void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const FbS
         if (m_utf8mode) {
             Xutf8DrawString(d.display(), d.drawable(), m_fontset,
                             main_gc, x, y,
-                            text.data(), len);
+                            text, len);
         } else
 #endif //X_HAVE_UTF8_STRING
             {
-                string localestr = text;
-                localestr.erase(len, string::npos);
-                localestr = FbStringUtil::FbStrToLocale(localestr);
+                std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
                 XmbDrawString(d.display(), d.drawable(), m_fontset,
                               main_gc, x, y,
                               localestr.data(), localestr.size());
@@ -238,13 +179,11 @@ void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const FbS
     if (m_utf8mode) {
         Xutf8DrawString(dpy, canvas.drawable(), m_fontset,
                              font_gc.gc(), xpos, ypos,
-                             text.data(), len);
+                             text, len);
     } else
 #endif //X_HAVE_UTF8_STRING
     {
-        string localestr = text;
-        localestr.erase(len, string::npos);
-        localestr = FbStringUtil::FbStrToLocale(localestr);
+        std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
         XmbDrawString(dpy, canvas.drawable(), m_fontset,
                            font_gc.gc(), xpos, ypos,
                            localestr.data(), localestr.size());
@@ -257,15 +196,8 @@ void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const FbS
     XCopyGC(dpy, main_gc, GCForeground|GCBackground, my_gc);
 
     // vertical or upside down
-
     XSetFillStyle(dpy, my_gc, FillStippled);
-
-    // vertical or upside down
-
-    XSetFillStyle(dpy, my_gc, FillStippled);
-
     XSetStipple(dpy, my_gc, canvas.drawable());
-
     XSetTSOrigin(dpy, my_gc, 0, 0);
 
     XFillRectangle(dpy, d.drawable(), my_gc, 0, 0,
@@ -276,25 +208,21 @@ void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const FbS
 
 }
 
-unsigned int XmbFontImp::textWidth(const FbString &text, unsigned int len) const {
+unsigned int XmbFontImp::textWidth(const char* text, unsigned int len) const {
+
     if (m_fontset == 0)
         return 0;
 
     XRectangle ink, logical;
 #ifdef X_HAVE_UTF8_STRING
     if (m_utf8mode) {
-        Xutf8TextExtents(m_fontset, text.data(), len,
-                         &ink, &logical);
+        Xutf8TextExtents(m_fontset, text, len, &ink, &logical);
         if (logical.width != 0)
             return logical.width;
     }
 #endif // X_HAVE_UTF8_STRING
 
-    string localestr = text;
-    if (len > localestr.length())
-        len = localestr.length();
-    localestr.erase(len, string::npos);
-    localestr = FbStringUtil::FbStrToLocale(localestr);
+    std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
     XmbTextExtents(m_fontset, localestr.data(), localestr.size(),
                    &ink, &logical);
     return logical.width;
